@@ -1,6 +1,7 @@
 #! /usr/bin/env python
 import argparse
 import logging
+import requests
 import signal
 
 from os import kill
@@ -8,11 +9,11 @@ from os.path import abspath, dirname, join
 
 from subprocess import PIPE, STDOUT, Popen
 from time import sleep
-from flask import Flask, redirect, render_template, Response
+from flask import Flask, redirect, render_template, Response, request
 from jinja2 import FileSystemLoader
 
-from touchandgo.helpers import get_interface, get_lock_diff, LOCKFILE, \
-    set_config_dir, get_free_port
+from touchandgo.helpers import get_interface, LOCKFILE, set_config_dir, \
+    get_free_port, is_process_running
 from touchandgo.history import History
 from touchandgo.lock import Lock
 from touchandgo.logger import log_set_up
@@ -33,33 +34,51 @@ def serve(py_exec=None):
     template_path = join(dirname(__file__), "templates")
     app.jinja_loader = FileSystemLoader(template_path)
 
+    def get_torrent(name, season=None, episode=None):
+        interface = get_interface()
+        path = abspath(__file__)
+        dir_path = dirname(path)
+        exec_ = join(dir_path, "__init__.py")
+        command = '%s %s \"%s\" ' % (py_exec, exec_, name)
+        if season is not None:
+            command += "%s %s " % (season, episode)
+        lock = Lock(LOCKFILE)
+        if lock.is_same_file(name, season, episode) and \
+                is_process_running(lock.get_pid()):
+            data = lock._get_file_data()
+            port = data[4]
+        else:
+            port = get_free_port()
+            command += "--daemon --port %s " % port
+            log.debug(command)
+            process = Popen(command, shell=True, stdout=PIPE, stderr=STDOUT)
+            sleep(1)
+
+        redirect_url = "http://%s:%s" % (interface, port)
+        serving = False
+        while not serving:
+            try:
+                req = requests.get("%s/status" % redirect_url)
+                serving = True
+            except requests.ConnectionError, e:
+                sleep(1)
+        log.info("redirecting to %s" % redirect_url)
+        return redirect(redirect_url)
+
     @app.route("/favicon.ico")
-    def favicon(self):
+    def favicon():
         return ""
 
+    @app.route("/magnet")
+    def magnet():
+        # Maybe someone wanted to search the movie "magnet", who knows
+        return get_torrent(request.args.get('m', 'magnet'))
+
     @app.route("/<name>")
     @app.route("/<name>/<season>/<episode>")
     def redirect_to(name, season=None, episode=None):
         log.info("Requesting %s %s %s", name, season, episode)
-        interface = get_interface()
-        path = abspath(__file__)
-        dir_path = dirname(path)
-        exec_ = join(dir_path, "__init__.py")
-
-        port = "8890"#get_free_port()
-
-        command = '%s %s \"%s\" ' % (py_exec, exec_, name)
-        if season is not None:
-            command += "%s %s " % (season, episode)
-        command += "--daemon --port %s " % port
-        log.debug(command)
-        process = Popen(command, shell=True, stdout=PIPE, stderr=STDOUT)
-        sleep(1)
-        while get_lock_diff() < 30:
-            sleep(1)
-        redirect_url = "http://%s:%s" % (interface, port)
-        log.info("redirecting to %s" % redirect_url)
-        return redirect(redirect_url)
+        return get_torrent(name, season, episode)
 
     @app.route("/l/<name>/<season>/<episode>")
     @app.route("/l/<name>/<season>/<episode>/")
@@ -77,7 +96,10 @@ def serve(py_exec=None):
     @app.route("/kill")
     def kill_():
         lock = Lock(LOCKFILE)
-        kill(lock.get_pid(), signal.SIGQUIT)
+        try:
+            kill(lock.get_pid(), signal.SIGQUIT)
+        except Exception, e:
+            pass
         return "OK"
 
     @app.route("/")
@@ -85,17 +107,19 @@ def serve(py_exec=None):
         series = []
         keys = []
         items = History.many(sorted="-date,-season,-episode")
+
+        # Filter duplicate entries in the history list
         for item in items:
             key = (item.name, item.season is not None)
             if key not in keys:
                 series.append(item)
                 keys.append(key)
 
-        return render_template("main.html", items=series)
-
+        return render_template("main.html", items=series, magnet=request.args.get('m', ''))
 
     set_config_dir()
     app.debug = True
+    kill_()
     app.run(host="0.0.0.0")
 
 
